@@ -14,10 +14,10 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import time
 from collections import OrderedDict
 from typing import TYPE_CHECKING, Any, Optional, List, Dict, Tuple
 
-import time
 from iconcommons.logger import Logger
 
 from iconservice.iiss.listener import EngineListener as IISSEngineListener
@@ -31,6 +31,7 @@ from ..base.exception import (
     InvalidParamsException, InvalidRequestException, OutOfBalanceException, FatalException,
     InternalServiceErrorException
 )
+from ..base.transaction import Transaction
 from ..icon_constant import ISCORE_EXCHANGE_RATE, IISS_MAX_REWARD_RATE, \
     IconScoreContextType, IISS_LOG_TAG, ROLLBACK_LOG_TAG, RCCalculateResult, INVALID_CLAIM_TX, Revision, \
     RevisionChangedFlag
@@ -51,7 +52,6 @@ if TYPE_CHECKING:
     from ..icx import IcxStorage
     from ..prep.data import Term
     from ..base.block import Block
-    from ..base.transaction import Transaction
 
 _TAG = IISS_LOG_TAG
 
@@ -233,11 +233,13 @@ class Engine(EngineBase):
                                                   ready_callback=self.ready_callback,
                                                   ipc_timeout=ipc_timeout,
                                                   icon_rc_path=icon_rc_path)
-        self._reward_calc_proxy.open(log_dir=log_dir,
-                                     sock_path=socket_path,
-                                     iiss_db_path=data_path,
-                                     icon_rc_monitor=icon_rc_monitor)
-        self._reward_calc_proxy.start()
+        self._reward_calc_proxy.open(sock_path=socket_path)
+        self._reward_calc_proxy.start(
+            log_dir=log_dir,
+            sock_path=socket_path,
+            iiss_db_path=data_path,
+            icon_rc_monitor=icon_rc_monitor
+        )
 
     def _close_reward_calc_proxy(self):
         self._reward_calc_proxy.stop()
@@ -714,8 +716,8 @@ class Engine(EngineBase):
         if not isinstance(address, Address):
             raise InvalidParamsException(f"Invalid address: {address}")
 
-        # TODO: error handling
-        iscore, block_height = self._reward_calc_proxy.query_iscore(address)
+        tx_hash = context.tx.hash if isinstance(context.tx, Transaction) else None
+        iscore, block_height = self._reward_calc_proxy.query_iscore(address, tx_hash)
 
         data = {
             "iscore": iscore,
@@ -958,3 +960,11 @@ class Engine(EngineBase):
 
     def get_reward_calculator_commit_block(self) -> Optional[Tuple[int, bytes]]:
         return self._reward_calc_proxy.get_commit_block()
+
+    def send_start_block(self, block_height: int, block_hash: bytes):
+        ret_block_height, ret_block_hash = \
+            self._reward_calc_proxy.start_block(block_height, block_hash)
+        if block_height == ret_block_height and block_hash == ret_block_hash:
+            return
+
+        raise InternalServiceErrorException("Failed to send start_block to reward_calculator")
